@@ -67,6 +67,7 @@ static int show_hdr = 1;
 static int all_ve = 0;
 static int only_stopped_ve = 0;
 static int with_names = 0;
+static long __clk_tck = -1;
 
 char logbuf[32];
 char *plogbuf = logbuf;
@@ -139,11 +140,35 @@ static void print_status(struct Cveinfo *p, int index)
 
 static void print_laverage(struct Cveinfo *p, int index)
 {
-	if (p->la == NULL)
+	if (p->cpustat == NULL)
 		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%14s", "-");
 	else
 		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer, "%1.2f/%1.2f/%1.2f",
-			p->la->la[0], p->la->la[1], p->la->la[2]);
+			p->cpustat->la[0], p->cpustat->la[1], p->cpustat->la[2]);
+}
+
+static void print_uptime(struct Cveinfo *p, int index)
+{
+	if (p->cpustat == NULL)
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer,
+				"%15s", "-");
+	else
+	{
+		unsigned int days, hours, min, secs;
+
+		days  = (unsigned int)(p->cpustat->uptime / (60 * 60 * 24));
+		min = (unsigned int)(p->cpustat->uptime / 60);
+		hours = min / 60;
+		hours = hours % 24;
+		min = min % 60;
+		secs = (unsigned int)(p->cpustat->uptime -
+				(60ull * min + 60ull * 60 * hours +
+				 60ull * 60 * 24 * days));
+
+		p_outbuffer += snprintf(p_outbuffer, e_buf - p_outbuffer,
+				"%.3dd%.2dh:%.2dm:%.2ds",
+				days, hours, min, secs);
+	}
 }
 
 static void print_cpulimit(struct Cveinfo *p, int index)
@@ -230,7 +255,7 @@ PRINT_DQ(diskinodes)
 
 /* Sort functions */
 
-inline int check_empty_param(const void *val1, const void *val2)
+static inline int check_empty_param(const void *val1, const void *val2)
 {
 	if (val1 == NULL && val2 == NULL)
 		return 0;
@@ -241,29 +266,40 @@ inline int check_empty_param(const void *val1, const void *val2)
 	return 2;
 }
 
-int none_sort_fn(const void *val1, const void *val2)
+static int none_sort_fn(const void *val1, const void *val2)
 {
 	return 0;
 }
 
-int laverage_sort_fn(const void *val1, const void *val2)
+static int laverage_sort_fn(const void *val1, const void *val2)
 {
-	const struct Cla *la1 = ((const struct Cveinfo *)val1)->la;
-	const struct Cla *la2 = ((const struct Cveinfo *)val2)->la;
+	const struct Ccpustat *st1 = ((const struct Cveinfo *)val1)->cpustat;
+	const struct Ccpustat *st2 = ((const struct Cveinfo *)val2)->cpustat;
 	int res;
 
-	if ((res = check_empty_param(la1, la2)) == 2)
+	if ((res = check_empty_param(st1, st2)) == 2)
 		return res;
-	res = (la1->la[0] - la2->la[0]) * 100;
+	res = (st1->la[0] - st2->la[0]) * 100;
 	if (res != 0)
 		return res;
-	res = (la1->la[1] - la2->la[1]) * 100;
+	res = (st1->la[1] - st2->la[1]) * 100;
 	if (res != 0)
 		return res;
-	return (la1->la[2] - la2->la[2]) * 100;
+	return (st1->la[2] - st2->la[2]) * 100;
 }
 
-int id_sort_fn(const void *val1, const void *val2)
+static int uptime_sort_fn(const void *val1, const void *val2)
+{
+	struct Ccpustat *st1 = ((const struct Cveinfo *)val1)->cpustat;
+	struct Ccpustat *st2 = ((const struct Cveinfo *)val2)->cpustat;
+	int res;
+
+	if ((res = check_empty_param(st1, st2)) != 2)
+		return res;
+	return (st2->uptime - st1->uptime);
+}
+
+static int id_sort_fn(const void *val1, const void *val2)
 {
 	int ret;
 	ret = (((const struct Cveinfo*)val1)->veid >
@@ -271,7 +307,7 @@ int id_sort_fn(const void *val1, const void *val2)
 	return ret;
 }
 
-int status_sort_fn(const void *val1, const void *val2)
+static int status_sort_fn(const void *val1, const void *val2)
 {
 	int res;
 	res = ((const struct Cveinfo*)val1)->status -
@@ -281,7 +317,7 @@ int status_sort_fn(const void *val1, const void *val2)
 	return res;
 }
 
-int bootorder_sort_fn(const void *val1, const void *val2)
+static int bootorder_sort_fn(const void *val1, const void *val2)
 {
 	int ret;
 	unsigned long *r1 = ((const struct Cveinfo*)val1)->bootorder;
@@ -303,14 +339,14 @@ int bootorder_sort_fn(const void *val1, const void *val2)
 	return (*r1 > *r2);
 }
 
-int ioprio_sort_fn(const void *val1, const void *val2)
+static int ioprio_sort_fn(const void *val1, const void *val2)
 {
 	return ((const struct Cveinfo *)val1)->io.ioprio >
 		((const struct Cveinfo *)val2)->io.ioprio;
 }
 
 #define SORT_STR_FN(name)						\
-int name ## _sort_fn(const void *val1, const void *val2)		\
+static int name ## _sort_fn(const void *val1, const void *val2)		\
 {									\
 	const char *h1 = ((const struct Cveinfo*)val1)->name;		\
 	const char *h2 = ((const struct Cveinfo*)val2)->name;		\
@@ -327,7 +363,7 @@ SORT_STR_FN(ostemplate)
 SORT_STR_FN(ip)
 
 #define SORT_UL_RES(fn, res, name, index)				\
-int fn(const void *val1, const void *val2)				\
+static int fn(const void *val1, const void *val2)			\
 {									\
 	const struct C ## res *r1 = ((const struct Cveinfo *)val1)->res;\
 	const struct C ## res *r2 = ((const struct Cveinfo *)val2)->res;\
@@ -377,7 +413,7 @@ SORT_DQ(diskinodes)
 SORT_UL_RES(cpulimit_sort_fn, cpu, limit, 0)
 SORT_UL_RES(cpuunits_sort_fn, cpu, limit, 1)
 
-struct Cfield field_names[] =
+static struct Cfield field_names[] =
 {
 /* ctid should have index 0 */
 {"ctid", "CTID", "%10s", 0, RES_NONE, print_veid, id_sort_fn},
@@ -495,10 +531,10 @@ struct Cfield field_names[] =
 {"numothersock.f", "NOTHSOCK.F", "%10s", 4, RES_UBC,print_ubc_numothersock, numothersock_f_sort_fn},
 
 {"dcachesize", "DCACHESZ", "%10s", 0, RES_UBC,print_ubc_dcachesize, dcachesize_h_sort_fn},
-{"dcachesize.m", "DCACHESZ.M", "%10s", 1, RES_UBC,print_ubc_dcachesize, dcachesize_h_sort_fn},
-{"dcachesize.b", "DCACHESZ.B", "%10s", 2, RES_UBC,print_ubc_dcachesize, dcachesize_h_sort_fn},
-{"dcachesize.l", "DCACHESZ.L", "%10s", 3, RES_UBC,print_ubc_dcachesize, dcachesize_h_sort_fn},
-{"dcachesize.f", "DCACHESZ.F", "%10s", 4, RES_UBC,print_ubc_dcachesize, dcachesize_h_sort_fn},
+{"dcachesize.m", "DCACHESZ.M", "%10s", 1, RES_UBC,print_ubc_dcachesize, dcachesize_m_sort_fn},
+{"dcachesize.b", "DCACHESZ.B", "%10s", 2, RES_UBC,print_ubc_dcachesize, dcachesize_b_sort_fn},
+{"dcachesize.l", "DCACHESZ.L", "%10s", 3, RES_UBC,print_ubc_dcachesize, dcachesize_l_sort_fn},
+{"dcachesize.f", "DCACHESZ.F", "%10s", 4, RES_UBC,print_ubc_dcachesize, dcachesize_f_sort_fn},
 
 {"numfile", "NFILE", "%10s", 0, RES_UBC,print_ubc_numfile, numfile_h_sort_fn},
 {"numfile.m", "NFILE.M", "%10s", 1, RES_UBC,print_ubc_numfile, numfile_m_sort_fn},
@@ -523,10 +559,11 @@ struct Cfield field_names[] =
 {"diskspace.h", "DSPACE.H", "%10s", 2, RES_QUOTA, print_diskspace, diskspace_h_sort_fn},
 
 {"diskinodes", "DINODES", "%10s", 0, RES_QUOTA, print_diskinodes, diskinodes_u_sort_fn},
-{"diskinodes.s", "DINODES.S", "%10s", 1, RES_QUOTA, print_diskinodes, diskinodes_u_sort_fn},
-{"diskinodes.h", "DINODES.H", "%10s", 2, RES_QUOTA, print_diskinodes, diskinodes_u_sort_fn},
+{"diskinodes.s", "DINODES.S", "%10s", 1, RES_QUOTA, print_diskinodes, diskinodes_s_sort_fn},
+{"diskinodes.h", "DINODES.H", "%10s", 2, RES_QUOTA, print_diskinodes, diskinodes_h_sort_fn},
 
-{"laverage", "LAVERAGE", "%14s", 0, RES_LA, print_laverage, laverage_sort_fn},
+{"laverage", "LAVERAGE", "%14s", 0, RES_CPUSTAT, print_laverage, laverage_sort_fn},
+{"uptime", "UPTIME", "%15s", 1, RES_CPUSTAT, print_uptime, uptime_sort_fn},
 
 {"cpulimit", "CPULIM", "%7s", 0, RES_CPU, print_cpulimit, cpulimit_sort_fn},
 {"cpuunits", "CPUUNI", "%7s", 1, RES_CPU, print_cpulimit, cpuunits_sort_fn},
@@ -538,7 +575,7 @@ struct Cfield field_names[] =
 	print_bootorder, bootorder_sort_fn},
 };
 
-void *x_malloc(int size)
+static void *x_malloc(int size)
 {
 	void *p;
 	if ((p = malloc(size)) == NULL) {
@@ -548,7 +585,7 @@ void *x_malloc(int size)
 	return p;
 }
 
-void *x_realloc(void *ptr, int size)
+static void *x_realloc(void *ptr, int size)
 {
 	void *tmp;
 
@@ -559,7 +596,7 @@ void *x_realloc(void *ptr, int size)
 	return tmp;
 }
 
-void usage()
+static void usage()
 {
 	fprintf(stderr, "\n"
 "Usage:	vzlist	[-a|-S] [-n] [-H] [-o field[,field...]] [-s [-]field]\n"
@@ -581,17 +618,17 @@ void usage()
 	);
 }
 
-int id_search_fn(const void* val1, const void* val2)
+static int id_search_fn(const void* val1, const void* val2)
 {
 	return (*(const int *)val1 - ((const struct Cveinfo*)val2)->veid);
 }
 
-int veid_search_fn(const void* val1, const void* val2)
+static int veid_search_fn(const void* val1, const void* val2)
 {
 	return (*(const int *)val1 - *(const int *)val2);
 }
 
-char* trim_eol_space(char *sp, char *ep)
+static char* trim_eol_space(char *sp, char *ep)
 {
 /*	if (ep == NULL)
 		ep = sp + strlen(sp); */
@@ -602,7 +639,7 @@ char* trim_eol_space(char *sp, char *ep)
 	return sp;
 }
 
-void print_hdr()
+static void print_hdr()
 {
 	struct Cfield_order *p;
 	int f;
@@ -625,7 +662,7 @@ void print_hdr()
 	1 - match
 	0 - do not match
 */
-inline int check_pattern(char *str, char *pat)
+static inline int check_pattern(char *str, char *pat)
 {
 	if (pat == NULL)
 		return 1;
@@ -634,7 +671,7 @@ inline int check_pattern(char *str, char *pat)
 	return !fnmatch(pat, str, 0);
 }
 
-void filter_by_hostname()
+static void filter_by_hostname()
 {
 	int i;
 
@@ -644,7 +681,7 @@ void filter_by_hostname()
 	}
 }
 
-void filter_by_name()
+static void filter_by_name()
 {
 	int i;
 
@@ -654,7 +691,7 @@ void filter_by_name()
 	}
 }
 
-void filter_by_description()
+static void filter_by_description()
 {
 	int i;
 
@@ -664,7 +701,7 @@ void filter_by_description()
 	}
 }
 
-void print_ve()
+static void print_ve()
 {
 	struct Cfield_order *p;
 	int i, f, idx;
@@ -703,7 +740,7 @@ void print_ve()
 	}
 }
 
-void add_elem(struct Cveinfo *ve)
+static void add_elem(struct Cveinfo *ve)
 {
 	veinfo = (struct Cveinfo *)x_realloc(veinfo,
 				sizeof(struct Cveinfo) * ++n_veinfo);
@@ -711,13 +748,13 @@ void add_elem(struct Cveinfo *ve)
 	return;
 }
 
-inline struct Cveinfo *find_ve(int veid)
+static inline struct Cveinfo *find_ve(int veid)
 {
 	return (struct Cveinfo *) bsearch(&veid, veinfo, n_veinfo,
 			sizeof(struct Cveinfo), id_search_fn);
 }
 
-void update_ve(int veid, char *ip, int status)
+static void update_ve(int veid, char *ip, int status)
 {
 	struct Cveinfo *tmp, ve;
 
@@ -740,7 +777,7 @@ void update_ve(int veid, char *ip, int status)
 	return;
 }
 
-void update_ubc(int veid, struct Cubc *ubc)
+static void update_ubc(int veid, struct Cubc *ubc)
 {
 	struct Cveinfo *tmp;
 
@@ -749,7 +786,7 @@ void update_ubc(int veid, struct Cubc *ubc)
 	return ;
 }
 
-void update_quota(int veid, struct Cquota *quota)
+static void update_quota(int veid, struct Cquota *quota)
 {
 	struct Cveinfo *tmp;
 
@@ -760,7 +797,7 @@ void update_quota(int veid, struct Cquota *quota)
 	return;
 }
 
-void update_cpu(int veid, unsigned long limit, unsigned long units)
+static void update_cpu(int veid, unsigned long limit, unsigned long units)
 {
 	struct Cveinfo *tmp;
 	struct Ccpu *cpu;
@@ -774,31 +811,17 @@ void update_cpu(int veid, unsigned long limit, unsigned long units)
 	return;
 }
 
-void update_la(int veid, struct Cla *la)
+static void update_cpustat(int veid, struct Ccpustat *st)
 {
 	struct Cveinfo *tmp;
 
 	if ((tmp = find_ve(veid)) == NULL)
 		return;
-	tmp->la = x_malloc(sizeof(*la));
-	memcpy(tmp->la, la, sizeof(*la));
+	tmp->cpustat = x_malloc(sizeof(*st));
+	memcpy(tmp->cpustat, st, sizeof(*st));
 	return;
 }
 
-char *parse_var(char *var)
-{
-	char *sp, *ep;
-
-	if (var == NULL)
-		return NULL;
-	sp = var;
-	while (*sp && (*sp == '"' || isspace(*sp))) sp++;
-	ep = var + strlen(var) - 1;
-	while (ep > sp && (*ep == '"' || isspace(*ep))) *ep-- = 0;
-
-	return strdup(sp);
-}
-
 #define MERGE_QUOTA(name, quota, dq)				\
 do {								\
 	if (dq.name != NULL) {					\
@@ -807,7 +830,7 @@ do {								\
 	}							\
 } while(0);
 
-void merge_conf(struct Cveinfo *ve, vps_res *res)
+static void merge_conf(struct Cveinfo *ve, vps_res *res)
 {
 	if (ve->ubc == NULL) {
 		ve->ubc = x_malloc(sizeof(struct Cubc));
@@ -890,7 +913,7 @@ do {								\
 	ve->io.ioprio = res->io.ioprio;
 }
 
-int read_ves_param()
+static int read_ves_param()
 {
 	int i;
 	char buf[128];
@@ -929,7 +952,7 @@ int read_ves_param()
 }
 
 
-int check_veid_restr(int veid)
+static int check_veid_restr(int veid)
 {
 	if (g_ve_list == NULL)
 		return 1;
@@ -948,7 +971,7 @@ do {						\
 	}					\
 } while(0);					\
 
-int get_ub()
+static int get_ub()
 {
 	char buf[256];
 	int veid, prev_veid;
@@ -1035,40 +1058,14 @@ int get_ub()
 	return 0;
 }
 
-char *remove_sp(char *str)
-{
-	char *sp, *ep, *tp;
-	int skip;
-
-	if (str == NULL)
-		return NULL;
-	tp = str;
-	sp = tp;
-	ep = str + strlen(str);
-	skip = 0;
-	while (sp < ep) {
-		*tp = *sp;
-		if (isspace(*sp)) {
-			*tp = ' ';
-			skip++;
-		} else
-			skip = 0;
-		if (skip <= 1)
-			tp++;
-		sp++;
-	}
-	*tp = 0;
-	return strdup(str);
-}
-
-char *invert_ip(char *ips)
+static char *invert_ip(char *ips)
 {
 	char *tmp, *p, *ep, *tp;
 	size_t len;
 	int rc;
 	unsigned int ip[4];
 	int family;
-	char ip_str[64];
+	char ip_str[INET6_ADDRSTRLEN];
 
 	if (ips == NULL)
 		return NULL;
@@ -1233,7 +1230,7 @@ error:
 }
 #endif
 
-int get_run_quota_stat()
+static int get_run_quota_stat()
 {
 	unsigned long usage, softlimit, hardlimit, time, exp;
 	int veid = 0, prev_veid = 0;
@@ -1279,73 +1276,35 @@ int get_run_quota_stat()
 	return 0;
 }
 
-int get_stop_quota_stat(int veid)
+static long get_clk_tck()
 {
-	char buf[255];
-	char res[16];
-	FILE *fp;
-	unsigned long usage, softlimit, hardlimit;
-	int status;
-	struct Cquota quota;
-
-	snprintf(buf, sizeof(buf), VZQUOTA " show %d 2>/dev/null", veid);
-	if ((fp = popen(buf, "r")) == NULL)
-		return 1;
-	memset(&quota, 0, sizeof(quota));
-	while(!feof(fp)) {
-		if (fgets(buf, sizeof(buf), fp) == NULL)
-			break;
-		if (sscanf(buf, "%15s %lu %lu %lu", res, &usage,
-				&softlimit, &hardlimit) != 4)
-		{
-			continue;
-		}
-		if (!strcmp(res, "1k-blocks")) {
-			quota.diskspace[0] = usage;
-			quota.diskspace[1] = softlimit;
-			quota.diskspace[2] = hardlimit;
-		} else if (!strcmp(res, "inodes")) {
-			quota.diskinodes[0] = usage;
-			quota.diskinodes[1] = softlimit;
-			quota.diskinodes[2] = hardlimit;
-		}
-	}
-	status = pclose(fp);
-	if (WEXITSTATUS(status))
-		return 1;
-	update_quota(veid, &quota);
-	return 0;
+	if (__clk_tck != -1)
+		return __clk_tck;
+	__clk_tck = sysconf(_SC_CLK_TCK);
+	return __clk_tck;
 }
 
-int get_stop_quota_stats()
-{
-	int i;
-	for (i = 0; i < n_veinfo; i++) {
-		if (veinfo[i].status == VE_STOPPED && !veinfo[i].hide)
-			get_stop_quota_stat(veinfo[i].veid);
-	}
-	return 0;
-}
-
-int get_ve_la(int veid)
+static int get_ve_cpustat(int veid)
 {
 	struct vz_cpu_stat stat;
 	struct vzctl_cpustatctl statctl;
-	struct Cla la;
+	struct Ccpustat st;
 
 	statctl.veid = veid;
 	statctl.cpustat = &stat;
 	if (ioctl(vzctlfd, VZCTL_GET_CPU_STAT, &statctl) != 0)
 		return 1;
-	la.la[0] = stat.avenrun[0].val_int + (stat.avenrun[0].val_frac * 0.01);
-	la.la[1] = stat.avenrun[1].val_int + (stat.avenrun[1].val_frac * 0.01);
-	la.la[2] = stat.avenrun[2].val_int + (stat.avenrun[2].val_frac * 0.01);
+	st.la[0] = stat.avenrun[0].val_int + (stat.avenrun[0].val_frac * 0.01);
+	st.la[1] = stat.avenrun[1].val_int + (stat.avenrun[1].val_frac * 0.01);
+	st.la[2] = stat.avenrun[2].val_int + (stat.avenrun[2].val_frac * 0.01);
 
-	update_la(veid, &la);
+	st.uptime = (float) stat.uptime_jif / get_clk_tck();
+
+	update_cpustat(veid, &st);
 	return 0;
 }
 
-int get_ves_la()
+static int get_ves_cpustat()
 {
 	int i;
 
@@ -1354,13 +1313,13 @@ int get_ves_la()
 	for (i = 0; i < n_veinfo; i++) {
 		if (veinfo[i].hide)
 			continue;
-		get_ve_la(veinfo[i].veid);
+		get_ve_cpustat(veinfo[i].veid);
 	}
 	close(vzctlfd);
 	return 0;
 }
 
-int get_mounted_status()
+static int get_mounted_status()
 {
 	int i;
 	char buf[512];
@@ -1385,7 +1344,7 @@ int get_mounted_status()
 	return 0;
 }
 
-int get_ves_cpu()
+static int get_ves_cpu()
 {
 	unsigned long tmp;
 	int veid, id, weight, rate;
@@ -1415,7 +1374,7 @@ int get_ves_cpu()
 	return 0;
 }
 
-int get_ve_list()
+static int get_ve_list()
 {
 	DIR *dp;
 	struct dirent *ep;
@@ -1444,20 +1403,20 @@ int get_ve_list()
 	return 0;
 }
 
-int search_field(char *name)
+static int search_field(char *name)
 {
 	unsigned int i;
 
 	if (name == NULL)
 		return -1;
-	for (i = 0; i < sizeof(field_names) / sizeof(*field_names); i++) {
+	for (i = 0; i < ARRAY_SIZE(field_names); i++) {
 		if (!strcmp(name, field_names[i].name))
 			return i;
 	}
 	return -1;
 }
 
-int build_field_order(char *fields)
+static int build_field_order(char *fields)
 {
 	struct Cfield_order *tmp, *prev = NULL;
 	char *sp, *ep, *p;
@@ -1506,7 +1465,7 @@ static inline int check_param(int res_type)
 	return 0;
 }
 
-int collect()
+static int collect()
 {
 	int update = 0;
 	int ret;
@@ -1525,8 +1484,8 @@ int collect()
 	}
 	if (check_param(RES_QUOTA))
 		get_run_quota_stat();
-	if (check_param(RES_LA))
-		get_ves_la();
+	if (check_param(RES_CPUSTAT))
+		get_ves_cpustat();
 	if (check_param(RES_CPU))
 		if (!only_stopped_ve && (ret = get_ves_cpu()))
 			return ret;
@@ -1541,17 +1500,17 @@ int collect()
 	return 0;
 }
 
-void print_names()
+static void print_names()
 {
 	unsigned int i;
 
-	for (i = 0; i < sizeof(field_names) / sizeof(*field_names); i++)
+	for (i = 0; i < ARRAY_SIZE(field_names); i++)
 		printf("%-15s %-15s\n", field_names[i].name,
 					field_names[i].hdr);
 	return;
 }
 
-void free_veinfo()
+static void free_veinfo()
 {
 	int i;
 
@@ -1570,8 +1529,8 @@ void free_veinfo()
 			free(veinfo[i].ubc);
 		if (veinfo[i].quota != NULL)
 			free(veinfo[i].quota);
-		if (veinfo[i].la != NULL)
-			free(veinfo[i].la);
+		if (veinfo[i].cpustat != NULL)
+			free(veinfo[i].cpustat);
 		if (veinfo[i].cpu != NULL)
 			free(veinfo[i].cpu);
 		if (veinfo[i].ve_root != NULL)
